@@ -256,6 +256,8 @@ static void hexdump(void *buf, int size) {
 static void zeemote_free(zeemote_t *zeemote) {  
   pthread_mutex_lock(&zeemote->mutex);
 
+  pthread_cond_broadcast(&zeemote->cond);
+
   zeemote_t **zeemoteP = &zeemote_list;
   while(*zeemoteP && *zeemoteP != zeemote)
     zeemoteP = &(*zeemoteP)->next;
@@ -351,6 +353,9 @@ static void *zeemote_thread(void *data) {
 			"ZEEMOTE_STICK\n", hdr.length);
 		read_num(zeemote, data.dummy, hdr.length - 2, 0);
 	      }
+
+	      pthread_cond_broadcast(&zeemote->cond);
+
 	      break;
 	      
 	    case ZEEMOTE_BATTERY:
@@ -365,6 +370,9 @@ static void *zeemote_thread(void *data) {
 			"in ZEEMOTE_BATTERY\n", hdr.length);
 		read_num(zeemote, data.dummy, hdr.length - 2, 0);
 	      }
+
+	      pthread_cond_broadcast(&zeemote->cond);
+
 	      break;
 	      
 	    case ZEEMOTE_BUTTONS:
@@ -385,6 +393,9 @@ static void *zeemote_thread(void *data) {
 			hdr.length);
 		read_num(zeemote, data.dummy, hdr.length - 2, 0);
 	      }
+
+	      pthread_cond_broadcast(&zeemote->cond);
+
 	      break;
 	      
 	    default:
@@ -402,6 +413,7 @@ static void *zeemote_thread(void *data) {
 #endif
 	      break;
 	    }
+	        
 	    pthread_mutex_unlock(&zeemote->mutex);
 	  }
 	}
@@ -439,6 +451,8 @@ zeemote_t *zeemote_connect(bdaddr_t *bdaddr) {
   zeemote->state[0].state = ZEEMOTE_STATE_UNKNOWN;
 
   pthread_mutex_init(&zeemote->mutex, NULL);
+  
+  pthread_cond_init(&zeemote->cond, NULL);
 
   /* fork a handler thread */
   if ( pthread_create(&zeemote->thread, NULL, 
@@ -467,3 +481,11 @@ zeemote_state_t *zeemote_get_state(zeemote_t *zeemote) {
 
   return &zeemote->state[1];
 }
+
+void wait_for_data(zeemote_t *zeemote) {  
+  pthread_mutex_lock(&zeemote->mutex);
+  
+  pthread_cond_wait(&zeemote->cond, &zeemote->mutex);
+
+  pthread_mutex_unlock(&zeemote->mutex);
+}
